from django.template import Library
try:  # Django >=1.9
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse


register = Library()


@register.filter
def resize(value, size, root_name='resize'):
    """
    Generates the url for the resized image prefixing with prefix_path
    return string url
    """
    return reverse('imagefit_resize', kwargs=dict(
        path_name=root_name, format=size, url=value))


@register.filter
def media_resize(value, size):
    """
    Generates the url for the resized image prefixing with MEDIA_ROOT
    return string url
    """
    return resize(value, size, 'media_resize')


@register.filter
def static_resize(value, size):
    """
    Generates the url for the resized image prefixing with STATIC_ROOT
    return string url
    """
    return resize(value, size, 'static_resize')



from django.conf import settings
from appconf import AppConf

import tempfile
import os


class ImagefitConf(AppConf):

    #: dictionary of preset names that have width and height values
    IMAGEFIT_PRESETS = {
        'thumbnail': {'width': 80, 'height': 80, 'crop': True},
        'medium': {'width': 320, 'height': 240},
        'original': {},
    }

    #: dictionary of output formats depending on the output image extension.
    IMAGEFIT_EXT_TO_FORMAT = {
        '.jpg': 'jpeg', '.jpeg': 'jpeg'
    }

    #: default format for any missing extension in IMAGEFIT_EXT_TO_FORMAT
    #: do not fall-back to a default format but raise an exception if set to None
    IMAGEFIT_EXT_TO_FORMAT_DEFAULT = 'png'

    #: root path from where to read urls
    IMAGEFIT_ROOT = ''

    IMAGEFIT_CACHE_ENABLED = True
    IMAGEFIT_CACHE_BACKEND_NAME = 'imagefit'

    settings.CACHES[IMAGEFIT_CACHE_BACKEND_NAME] = {
        'BACKEND': 'django.core.cache.backends.filebased.FileBasedCache',
        'LOCATION': os.path.join(tempfile.gettempdir(), 'django_imagefit')
    }

    #: ConditionalGetMiddleware is required for browser caching
    # Django's middleware management (and some other things) has changed
    # since version 1.9.
    property_name = (hasattr(settings, 'MIDDLEWARE') and 'MIDDLEWARE') or 'MIDDLEWARE_CLASSES'
    middlewares = getattr(settings, property_name)
    if 'django.middleware.http.ConditionalGetMiddleware' not in middlewares:
        setattr(settings, property_name, middlewares + ('django.middleware.http.ConditionalGetMiddleware',))


def ext_to_format(filename):
    extension = os.path.splitext(filename)[1].lower()
    format = settings.IMAGEFIT_EXT_TO_FORMAT.get(extension, settings.IMAGEFIT_EXT_TO_FORMAT_DEFAULT)
    if not format:
        raise KeyError('Unknown image extension: {0}'.format(extension))
    return format



from __future__ import division
from imagefit.conf import ext_to_format, settings
from PIL import Image as PilImage

import mimetypes
try:
    import BytesIO
except ImportError:
    from io import BytesIO
import re
import os


class Image(object):
    """
    Represents an Image file on the system.
    """

    def __init__(self, path, cache=None, cached_name=None, *args, **kwargs):
        self.path = path
        self.pil = PilImage.open(path)
        self.cache = cache
        self.cached_name = cached_name

        # force RGB
        if self.pil.mode not in ('L', 'RGB', 'LA', 'RGBA'):
            self.pil = self.pil.convert('RGB')

    @property
    def mimetype(self):
        return mimetypes.guess_type(self.path)[0]

    @property
    def modified(self):
        return os.path.getmtime(self.path)

    @property
    def is_cached(self):
        return self.cache and self.cached_name in self.cache

    def resize(self, width=None, height=None):
        return self.pil.thumbnail(
            (int(width), int(height)),
            PilImage.ANTIALIAS)

    def crop(self, width=None, height=None):
        img_w, img_h = self.pil.size
        # don't crop an image than is smaller than requested size
        if img_w < width and img_h < height:
            return self.pil
        elif img_w < width:
            width = img_w
        elif img_h < height:
            height = img_h
        delta_w = img_w / width
        delta_h = img_h / height
        delta = delta_w if delta_w < delta_h else delta_h
        new_w = img_w / delta
        new_h = img_h / delta
        self.resize(new_w, new_h)
        box_diff = ((new_w - width) / 2, (new_h - height) / 2)
        box = (
            int(box_diff[0]), int(box_diff[1]), int(new_w - box_diff[0]),
            int(new_h - box_diff[1]))
        self.pil = self.pil.crop(box)
        return self.pil

    def render(self):
        """
        Renders the file content
        """
        if self.is_cached:
            return self.cache.get(self.cached_name)
        else:
            image_str = BytesIO()
            self.pil.save(image_str, ext_to_format(self.cached_name))
            return image_str.getvalue()

    def save(self):
        """
        Save the image to the cache if provided and not cached yet.
        """
        if self.cache and not self.is_cached:
            image_str = BytesIO()
            self.pil.save(image_str, ext_to_format(self.cached_name))
            self.cache.set(self.cached_name, image_str.getvalue())
            image_str.close()


class Presets(object):
    """
    Representation of an image format storage
    """

    @classmethod
    def get_all(cls):
        """
        Reads presets from settings
        """
        return getattr(settings, 'IMAGEFIT_PRESETS', {})

    @classmethod
    def get(cls, key, to_tuple=False):
        """
        Retrieves a specific preset by its name
        """
        preset = cls.get_all().get(key, None)
        return preset

    @classmethod
    def has(cls, key):
        """
        Checks if a preset exists
        """
        return key in cls.get_all()

    @classmethod
    def from_string(cls, string):
        """
        Converts a <width>x<height> into a {'width': <width>,
        'height': <height>} dict
        return dict or None
        """
        if re.match('(\d+)x(\d+),?(\w*)', string):
            sizes = [x for x in re.match(
                '(\d+)x(\d+)(,?[c|C]?)', string).groups()]
            return {
                'width': int(sizes[0]), 'height': int(sizes[1]),
                'crop': bool(sizes[2])}



from django.conf.urls import url
from . import views


urlpatterns = [
    url(
        r'^(?P<path_name>[\w_-]*)/(?P<format>[,\w-]+)/(?P<url>.*)/?$',
        views.resize,
        name="imagefit_resize"),
]



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


setup(
    name='django-imagefit',
    version='0.6.1',
    description='Render an optimized version of your original image on display. Ability to resize and crop.',
    long_description=open('README.md').read(),
    author='Vincent Agnano',
    author_email='vincent.agnano@scopyleft.fr',
    url='http://github.com/vinyll/django-imagefit',
    license='BSD',
    packages=find_packages(),
    zip_safe=False,
    install_requires=['django-appconf'],
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 3.3',
        'Topic :: Utilities',
    ]
)



from django.http import HttpResponse, HttpResponseNotModified
from django.core.exceptions import ImproperlyConfigured
from django.core.cache import caches
from django.utils.http import http_date
from django.views.static import was_modified_since

from imagefit.conf import settings
from imagefit.models import Image, Presets

import os
import stat
import time


cache = caches[settings.IMAGEFIT_CACHE_BACKEND_NAME]


def _image_response(image):
    response = HttpResponse(
        image.render(),
        image.mimetype
    )
    response['Last-Modified'] = http_date(image.modified)
    expire_time = getattr(settings, 'IMAGEFIT_EXPIRE_HEADER', 3600*24*30)
    response['Expires'] = http_date(time.time() + expire_time)
    return response


def resize(request, path_name, format, url):
    if path_name == 'static_resize':
        prefix = settings.STATIC_ROOT
    elif path_name == 'media_resize':
        prefix = settings.MEDIA_ROOT
    else:
        prefix = settings.IMAGEFIT_ROOT
    # remove prepending slash
    if url[0] == '/':
        url = url[1:]
    # generate Image instance
    image = Image(path=os.path.join(prefix, url))
    if not os.path.exists(image.path):
        return HttpResponse(status=404)

    statobj = os.stat(image.path)
    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
                              statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
        return HttpResponseNotModified(content_type=image.mimetype)

    if settings.IMAGEFIT_CACHE_ENABLED:
        image.cache = cache
        image.cached_name = request.META.get('PATH_INFO')
        # shortcut everything, render cached version
        if image.is_cached:
            return _image_response(image)

    # retrieve preset from format argument
    preset = Presets.get(format) or Presets.from_string(format)
    if not preset:
        raise ImproperlyConfigured(
            " \"%s\" is neither a \"WIDTHxHEIGHT\" format nor a key in " +
            "IMAGEFIT_PRESETS." % format
        )

    # Resize and cache image
    if preset.get('crop'):
        image.crop(preset.get('width'), preset.get('height'))
    else:
        image.resize(preset.get('width'), preset.get('height'))
    image.save()

    return _image_response(image)
